<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    customily-options
 * @subpackage customily-options/admin
 * @author     Customily
 */

const ORDERS_STATUS = array(
    'processing',
    'completed',
    'confirmed',
    'procesando',
    'completado',
    'confirmado',
    'confirmadocliente',
    'descargado-bot',
    'retirada',
    'pedido-fechado',
    'idw',
    'transporte'
);

class Customily_Options_Admin
{
    /**
     * Plugin api client
     *
     * @var Customily_Options_Api_Client
     */
    private $api_client;

    public function __construct()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/customily-options-api-client.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/customily-options-utils.php';
        $this->api_client = new Customily_Options_Api_Client(Customily_Options_Utils::get_site_url());
    }

    /**
     * Hook to add Customily Options item to WooCommerce admin menu.
     *
     * @since 1.0.0
     * @access public
     *
     */
    public function wc_customily_admin_menu()
    {
        if (!current_user_can('manage_woocommerce')) {
            return;
        }

        if (get_option('cl_hide_v2_admin_menu') == 'y') {
            return;
        }

        add_submenu_page(
            'woocommerce',
            CUSTOMILY_OPTIONS_ENV == 'beta' ? 'Customily_Beta' : 'Customily',
            CUSTOMILY_OPTIONS_ENV == 'beta' ? 'Customily_Beta' : 'Customily',
            'manage_woocommerce',
            CUSTOMILY_OPTIONS_ENV == 'beta' ? 'customily_beta' : 'customily',
            'load_customily_settings',
            2
        );

        function load_customily_settings()
        {
            include plugin_dir_path(__FILE__) . '/customily-options-admin-tab.php';
        }
    }

    /**
     * Hook to update info in app when a product is published.
     *
     * @since 1.0.0
     * @access public
     *
     * @param string $new_status The post new status.
     * @param string $old_status The post old status.
     * @param WP_Post $post The product being published.
     */
    public function wc_customily_publish_product($new_status, $old_status, $post)
    {
        if (
            $post->post_type == 'product' && (
                $new_status === 'private' ||
                $new_status === 'publish' ||
                $old_status === 'importing'
            )
        ) {
            $product = wc_get_product($post->ID);
            Customily_Options_Utils::add_log(
                "Customily Wordpress: Updating product {$product->get_name()}," .
                " old status: {$old_status} <-> new status: {$new_status}"
            );
            try {
                $result = $this->api_client->update_product_hook($post->ID);
                if (!empty($result)) {
                    Customily_Options_Utils::add_log(
                        "Customily Wordpress: Updated product {$product->get_name()}," .
                        " old status: {$old_status} <-> new status: {$new_status}"
                    );
                }
            } catch (Exception $e) {
                Customily_Options_Utils::add_log(
                    "Customily Wordpress: Error updating product {$product->get_name()}," .
                    " old status: {$old_status} <-> new status: {$new_status}"
                );
                Customily_Options_Utils::add_log($e->getTraceAsString());
            }
        }
    }

    /**
     * aaa
     * @param WC_Product $product
     * @param array $data
     */
    public function wc_customily_product_imported($product, $data)
    {
        if (is_a($product, 'WC_Product') && $product->get_type() != 'variation') {
            Customily_Options_Utils::add_log("Customily Wordpress: Updating imported product {$product->get_name()}");
            try {
                $result = $this->api_client->update_product_hook($product->get_id());
                if (!empty($result)) {
                    Customily_Options_Utils::add_log("Customily Wordpress: Updated imported product {$product->get_name()}");
                }
            } catch (Exception $e) {
                Customily_Options_Utils::add_log(
                    "Customily Wordpress: Error updating imported product {$product->get_name()}"
                );
                Customily_Options_Utils::add_log($e->getTraceAsString());
            }
        }
    }

    /**
     * Hook to remove info in app when a product is deleted.
     *
     * @since 1.0.0
     * @access public
     *
     * @param int $post_id The ID in the database table of the product being deleted.
     */
    public function wc_customily_before_delete_product($post_id)
    {
        global $post_type;
        if ($post_type == 'product') {
            $product = wc_get_product($post_id);
            if ($product) {
                if ($product->get_parent_id() == 0) {
                    Customily_Options_Utils::add_log("Customily Wordpress: Deleting product {$post_id} - {$product->get_name()}");
                    try {
                        $this->api_client->delete_product_hook($post_id);
                        Customily_Options_Utils::add_log("Customily Wordpress: Deleted product {$post_id} - {$product->get_name()}");
                    } catch (Exception $e) {
                        Customily_Options_Utils::add_log("Customily Wordpress: Error deleting product {$post_id} - {$product->get_name()}");
                        Customily_Options_Utils::add_log($e->getTraceAsString());
                    }
                }
            }
        }
    }

    /**
     * Hook for order status changed.
     * @param $order_id
     * @param $status_from
     * @param $status_to
     * @param $order
     * @return false
     */
    public function wc_customily_woocommerce_order_status_changed_hook($order_id, $status_from, $status_to, $order) {
        if (!$order_id) {
            return false;
        }

        Customily_Options_Utils::add_log("Customily Wordpress: order {$order_id} status changed from {$status_from} to {$status_to}");

        if (in_array($status_to, ORDERS_STATUS)) {
            require_once plugin_dir_path(dirname(__FILE__)) . 'public/customily-options-public.php';
            $plugin_public = new Customily_Options_Public();
            $plugin_public->wc_customily_notify_order_creation($order_id);
        }
    }

    /**
     * Hook to show personalization data in order details.
     *
     * @since 1.0.0
     * @access public
     *
     * @param WC_Order $order The woocommerce order
     */
    public function wc_customily_show_custom_data_in_order($order) {
        if (!class_exists('Customily')) {
            echo '<div class="order_data_column" style="width: 100%">';

            if ($order->meta_exists('Mobile')) {
                echo '<div class="address"><strong style="margin: 0.8em 0 0; padding: 0;">Mobile:</strong><span>' . esc_html($order->get_meta('Mobile')) . '</span></div>';
            }

            if ($order->meta_exists('OS')) {
                echo '<div class="address"><strong style="margin: 0.8em 0 0; padding: 0;">OS:</strong><span>' . esc_html($order->get_meta('OS')) . '</span></div>';
            }

            if ($order->meta_exists('Browser')) {
                echo '<div class="address"><strong style="margin: 0.8em 0 0; padding: 0;">Browser:</strong><span>' . esc_html($order->get_meta('Browser')) . '</span></div>';
            }

            if ($order->meta_exists('tracking_company')) {
                $tracking_company = $order->get_meta('tracking_company');
                $tracking_number = $order->get_meta('tracking_number');
                $tracking_url = $order->get_meta('tracking_url');
                if (!empty($tracking_url)) {
                    $tracking_number = '<a href="' . esc_url($tracking_url) . ' target="_blank">' . esc_html($tracking_number) . '</a>';
                }

                echo '<div class="address"><strong style="margin: 0.8em 0 0; padding: 0;">' . esc_html($tracking_company) . ' tracking number:</strong><span>' . $tracking_number . '</span></div>';
            }

            echo '</div>';
        }
    }

    /**
     * Show custom thumbnail on order
     *
     * @param $product_image
     * @param $item_id
     * @param $item
     * @return mixed|string
     * @throws Exception
     */
    public function wc_customily_show_custom_thumbnail_on_order($product_image, $item_id, $item) {
        if (!class_exists('Customily')) {
            $thumbnail_link = $item->get_meta('Thumbnail');
            if (!empty($thumbnail_link)) {
                if (substr_count($thumbnail_link, 'href') > 0) {
                    try {
                        $thumbnail_link = new SimpleXMLElement($thumbnail_link);
                        $thumbnail_link = $thumbnail_link['href'];
                    } catch (Exception $ex) {
                        // do nothing
                    }
                }
                $product_image = "<img src=\"{$thumbnail_link}\" style=\"object-fit: contain;\"/>";
            }
        }
        return $product_image;
    }

    /**
     *
     * @param $permission
     * @param $context
     * @param $zero
     * @param $object
     * @return bool|mixed
     */
    public function wc_customily_rest_check_permissions($permission, $context, $zero, $object) {
        // Disable authentication for specific IP addresses.
        $permitted_ips = array('52.53.105.196', '54.144.132.187');
        if (!empty($_SERVER['REMOTE_ADDR']) && in_array($_SERVER['REMOTE_ADDR'], $permitted_ips)) {
            return true;  // Allow access
        }

        return $permission;
    }

    public function wc_customily_register_custom_order_status()
    {
        register_post_status(
            'wc-pod-fulfilled',
            array(
                'label' => __('POD Fulfilled', 'customily-v2'),
                'public' => true,
                'exclude_from_search' => false,
                'show_in_admin_all_list' => true,
                'show_in_admin_status_list' => true,
                /* translators: %s: count */
                'label_count' => _n_noop(
                    'POD Fulfilled <span class="count">(%s)</span>',
                    'POD Fulfilled <span class="count">(%s)</span>',
                    'customily-v2'
                )
            )
        );
    }

    /**
     * Add custom order statuses
     *
     * @param $order_statuses
     * @return mixed
     */
    public function wc_customily_woocommerce_add_order_statuses($order_statuses) {
        $new_order_statuses = array();
        // add new order status before completed
        foreach ($order_statuses as $key => $status) {
            $new_order_statuses[$key] = $status;
            if ($key === 'wc-completed') {
                $new_order_statuses['wc-pod-fulfilled'] = __('POD Fulfilled', 'customily-v2');
            }
        }
        return $new_order_statuses;
    }
}