<?php
class Customily_Options_Orders_DB
{
    /**
     * Get order by id
     * @param $order_id
     * @param string $include_all_keys
     * @param string $production_file_full_url
     * @return array|false
     */
    public function get_order_by_id($order_id, $include_all_keys = '0', $production_file_full_url = '0')
    {
        require_once plugin_dir_path(dirname(__FILE__)) . '../includes/customily-options-utils.php';
        $order = wc_get_order($order_id);
        return $this->transform_order($order, $include_all_keys, $production_file_full_url);
    }

    /**
     * Get orders between ids
     * @param $from_id
     * @param $to
     * @param $include_all_keys
     * @return array
     */
    public function get_orders_between_ids($from_id, $to_id, $include_all_keys = '0') {
        $result = array();
        for ($id = $from_id; $id <= $to_id; $id++) {
            $order = $this->get_order_by_id($id, $include_all_keys);
            if ($order) {
                $result[] = $order;
            }
        }
        return $result;
    }

    /**
     * @param $from_date
     * @param $to_date
     * @param string $include_all_keys
     * @param string $production_file_full_url
     * @return array
     */
    public function get_orders_between_dates(
        $from_date,
        $to_date,
        $include_all_keys = '0',
        $production_file_full_url = '0'
    ) {
        $result = [];
        $orders = wc_get_orders(array(
                'limit' => -1,
                'type' => 'shop_order',
                'date_created' => "{$from_date}...{$to_date}"
            )
        );

        if (count($orders) > 0) {
            foreach ($orders as $order) {
                $result[] = $this->transform_order($order, $include_all_keys, $production_file_full_url);
            }
        }
        return $result;
    }

    /**
     * Get customily information
     * @param $order_id
     * @param $include_all_keys
     * @return array
     */
    public function get_customily_info($order_id, $include_all_keys = false) {
        $result = array(
            'order' => false,
            'personalization' => false
        );
        $order = wc_get_order($order_id);
        if (!empty($order)) {
            $result['order'] = $order;
            foreach ($order->get_items() as $item) {
                $meta_data = $item->get_meta_data();
                $cart_item_key = $item->get_meta('_cart_item_key');
                $personalization = [];
                foreach ($meta_data as $meta) {
                    if (
                        !$include_all_keys &&
                        ($meta->key == 'Preview' || substr_count($meta->key, '_') == 0)
                    ) {
                        continue;
                    }
                    $personalization[] = [
                        'key' => $meta->key,
                        'value' => $meta->value
                    ];
                }
                $result['personalization'] = array(
                    $cart_item_key => $personalization
                );
            }
        }
        return $result;
    }

    /**
     * @param $order_id
     * @param $cart_item_key
     * @param $meta_key
     * @param $meta_value
     * @return bool|null
     */
    public function update_order_item_meta($order_id, $cart_item_key, $meta_key, $meta_value) {
        $result = false;
        $order = wc_get_order($order_id);
        if (!empty($order)) {
            foreach ($order->get_items() as $key => $order_item) {
                if ($order_item->get_meta('_cart_item_key') == $cart_item_key) {
                    try {
                        wc_update_order_item_meta($key, $meta_key, $meta_value);
                        $result = true;
                    } catch (\Exception $e) {
                        Customily_Options_Utils::add_log($e->getTraceAsString());
                    }
                    break;
                }
            }
        }
        return $result;
    }

    /**
     * Create order fulfillment
     *
     * @param $line_items_by_fulfillment_order
     * @param $notify_customer
     * @param $tracking_info
     */
    public function create_order_fulfillment($line_items_by_fulfillment_order, $notify_customer, $tracking_info) {
        if (
            count($line_items_by_fulfillment_order) > 0 &&
            $line_items_by_fulfillment_order[0] &&
            $line_items_by_fulfillment_order[0]['fulfillmentOrderId']
        ) {
            $order_id = $line_items_by_fulfillment_order[0]['fulfillmentOrderId'];
            $order = wc_get_order($order_id);
            if (!empty($order)) {
                $order->set_status('pod-fulfilled');
                $order->add_meta_data('tracking_number', $tracking_info['number']);
                $order->add_meta_data('tracking_company', $tracking_info['company']);
                $order->add_meta_data('tracking_url', $tracking_info['url']);
                $order->save();
                return $order->get_id();
            }
        }
        return -1;
    }

    /**
     * Transform order data
     * @param WC_Order $order
     * @param $include_all_keys
     * @param $production_file_full_url
     * @return array|false
     */
    private function transform_order($order, $include_all_keys, $production_file_full_url)
    {
        if ($order) {
            $personalized_count = 0;
            $items = [];
            $currency = $order->get_currency();

            foreach ($order->get_items() as $key => $order_item) {
                $item = new WC_Order_Item_Product($order_item->get_id());
                $product_id = $item->get_product_id();
                $variant_id = $item->get_variation_id();
                $cart_item_key = $order_item->get_meta('_cart_item_key');
                $properties = [];
                $properties[] = [
                    'name' => 'cart_item_key',
                    'value' => $cart_item_key
                ];

                $product = $item->get_product();
                if ($product->is_type('variable')) {
                    $variation = new WC_Product_Variable($variant_id);
                }

                $file_numbers_array = ['1'];
                if ($order->meta_exists("file_numbers_{$cart_item_key}")) {
                    try{
                        $file_numbers = $order->get_meta("file_numbers_{$cart_item_key}");
                        $file_numbers_array = explode(',', $file_numbers);
                        $properties[] = [
                            'name' => '_customily-file-numbers',
                            'value' => $file_numbers
                        ];
                    } catch (Exception $ex) {
                        Customily_Options_Utils::add_log($ex->getTraceAsString());
                    }
                }

                foreach ($file_numbers_array as $file_number) {
                    $file_number = (int) $file_number;
                    $order_meta_suffix = '';
                    $customily_suffix = '';

                    // If the file number is 1, no suffix is used, this was done to maintain backward compatibility.
                    if ($file_number > 1) {
                        $order_meta_suffix = "{$file_number}_";
                        $customily_suffix = "-{$file_number}";
                    }

                    if ($order->meta_exists("personalization_id_{$order_meta_suffix}{$cart_item_key}")) {
                        $properties[] = [
                            'name' => "_customily-personalization-id{$customily_suffix}",
                            'value' => $order->get_meta("personalization_id_{$order_meta_suffix}{$cart_item_key}")
                        ];
                    }

                    if ($order->meta_exists("production_file_{$order_meta_suffix}{$cart_item_key}")) {
                        $personalized_count++;
                        try {
                            $production_file_url = $order->get_meta("production_file_{$order_meta_suffix}{$cart_item_key}");
                            $properties[] = [
                                'name' => "_customily-production-url{$customily_suffix}",
                                'value' => $production_file_full_url == '1'
                                    ? $production_file_url
                                    : pathinfo($production_file_url, PATHINFO_BASENAME)
                            ];
                            // for retro compatibility
                            $properties[] = [
                                'name' => "_customily-production-file{$customily_suffix}",
                                'value' => $production_file_full_url == '1'
                                    ? $production_file_url
                                    : pathinfo($production_file_url, PATHINFO_BASENAME)
                            ];
                        } catch (Exception $ex) {
                            Customily_Options_Utils::add_log($ex->getTraceAsString());
                        }
                    }

                    if ($order->meta_exists("production_file_renamed_{$order_meta_suffix}{$cart_item_key}")) {
                        $properties[] = [
                            'name' => "_customily-production-file-renamed{$customily_suffix}",
                            'value' => true
                        ];
                    }

                    if ($order->meta_exists("eps_name_{$order_meta_suffix}{$cart_item_key}")) {
                        $properties[] = [
                            'name' => "_customily-eps-name{$customily_suffix}",
                            'value' => $order->get_meta("eps_name_{$order_meta_suffix}{$cart_item_key}")
                        ];
                    }

                    if ($order->meta_exists("eps_placeholder_{$order_meta_suffix}{$cart_item_key}")) {
                        $properties[] = [
                            'name' => "_customily-eps-placeholder{$customily_suffix}",
                            'value' => $order->get_meta("eps_placeholder_{$order_meta_suffix}{$cart_item_key}")
                        ];
                    }
                }

                if (empty($product_id) || $product_id <= 0) {
                    $product_id = $product->get_id();
                }

                if (empty($variant_id) || $variant_id <= 0) {
                    $variant_id = $product_id;
                }

                if ($include_all_keys == '1') {
                    $meta_data = $order_item->get_meta_data();
                    foreach ($meta_data as $meta) {
                        if ($meta->key == '_cart_item_key') {
                            continue;
                        }

                        $key = $meta->key;
                        $value = $meta->value;

                        if ($key == 'Preview') {
                            $key = '_customily-preview';
                        } else if ($key == 'Thumbnail') {
                            $key = '_customily-thumb';
                        }

                        if ($key == '_customily-preview' || $key == '_customily-thumb') {
                            $value = Customily_Options_Utils::extract_url_from_link($value);
                        }

                        $properties[] = [
                            'name' => $key,
                            'value' => $value
                        ];
                    }
                }

                $parentProduct = wc_get_product($product_id);
                if ($parentProduct->meta_exists('pod_vendor')) {
                    $vendor = $parentProduct->get_meta('pod_vendor');
                } else {
                    $vendor = 'No vendor';
                }

                $price_set = [
                    'shop_money' => [
                        'amount' => number_format($item->get_total(), 2, '.', ''),
                        'currency_code' => $currency
                    ]
                ];

                $items[] = [
                    'id' => $order_item->get_id(),
                    'product_id' => $product_id,
                    'variant_id' => $variant_id,
                    'sku' => !empty($variation) ? $variation->get_sku() : $product->get_sku(),
                    'name' => $product->get_name(),
                    'price' => number_format(floatval($item->get_total()), 2, '.', ''),
                    'price_set' => $price_set,
                    'quantity' => $item->get_quantity(),
                    'variant_title' => !empty($variation) ? $variation->get_title() : $product->get_title(),
                    'properties' => $properties,
                    'vendor' => $vendor,
                ];
            }

            $billing_country = Customily_Options_Utils::get_country_name($order->get_billing_country());
            $shipping_country = Customily_Options_Utils::get_country_name($order->get_shipping_country());

            $total_price_set = [
                'shop_money' => [
                    'amount' => number_format($order->get_total(), 2, '.', ''),
                    'currency_code' => $currency
                ]
            ];

            $total_tax_set = [
                'shop_money' => [
                    'amount' => number_format($order->get_total_tax(), 2, '.', ''),
                    'currency_code' => $currency
                ]
            ];

            $subtotal_price_set = [
                'shop_money' => [
                    'amount' => number_format($order->get_subtotal(), 2, '.', ''),
                    'currency_code' => $currency
                ]
            ];

            $total_discounts = [
                'shop_money' => [
                    'amount' => number_format($order->get_total_discount(), 2, '.', ''),
                    'currency_code' => $currency
                ]
            ];

            $shipping_total = number_format(floatval($order->get_shipping_total()), 2, '.', '');
            $shipping_tax = number_format(floatval($order->get_shipping_tax()), 2, '.', '');

            $total_shipping_price_set = [
                'shop_money' => [
                    'amount' => floatval($shipping_total) + floatval($shipping_tax),
                    'currency_code' => $currency
                ]
            ];

            $result = array(
                'id' => $order->get_id(),
                'name' => $order->get_order_number(),
                'created_at' => $order->get_date_created()->__toString(),
                'customer' => [
                    "id" => $order->get_id(),
                    'email' => $order->get_billing_email(),
                    'first_name' => $order->get_billing_first_name(),
                    'last_name' => $order->get_billing_last_name(),
                    'phone' => $order->get_billing_phone(),
                    'state' => $order->get_billing_state(),
                    'tags' => '',
                    'created_at' => $order->get_date_created()->__toString(),
                    'updated_at' => $order->get_date_created()->__toString()
                ],
                'email' => $order->get_billing_email(),
                'currency' => $currency,
                'total_price_set' => $total_price_set,
                'total_tax_set' => $total_tax_set,
                'subtotal_price_set' => $subtotal_price_set,
                'total_discounts' => $total_discounts,
                'total_shipping_price_set' => $total_shipping_price_set,
                'billing_address' => [
                    'first_name' => $order->get_billing_first_name(),
                    'last_name' => $order->get_billing_last_name(),
                    'name' => '',
                    'address1' => $order->get_billing_address_1(),
                    'address2' => $order->get_billing_address_2(),
                    'city' => $order->get_billing_city(),
                    'company' => $order->get_billing_company(),
                    'province_code' => $order->get_billing_state(),
                    'province' => $order->get_billing_state(),
                    'country_code' => $order->get_billing_country(),
                    'country' => $billing_country,
                    'zip' => $order->get_billing_postcode(),
                    'phone' => $order->get_billing_phone()
                ],
                'shipping_address' => [
                    'first_name' => $order->get_shipping_first_name(),
                    'last_name' => $order->get_shipping_last_name(),
                    'name' => '',
                    'address1' => $order->get_shipping_address_1(),
                    'address2' => $order->get_shipping_address_2(),
                    'city' => $order->get_shipping_city(),
                    'company' => $order->get_shipping_company(),
                    'province_code' => $order->get_shipping_state(),
                    'province' => $order->get_shipping_state(),
                    'country_code' => $order->get_shipping_country(),
                    'country' => $shipping_country,
                    'zip' => $order->get_shipping_postcode(),
                    'phone' => $order->get_billing_phone()
                ],
                'line_items' => $items,
                'fulfillment_orders' => [],
                'shipping_lines' => [],
                'personalized_count' => $personalized_count
            );

            $updated_at = $order->get_date_modified();
            $result['updated_at'] = !empty($updated_at) ? $updated_at->__toString() : $result['created_at'];

            return $result;
        }
        return false;
    }
}