<?php
class Customily_Options_Orders_Notifications_DB
{
    private $orders_notifications_table_name = 'orders_notifications';

    /**
     * Customily_Options_Orders_Notifications_DB constructor.
     */
    public function __construct()
    {
        global $wpdb;
        require_once plugin_dir_path(dirname(__FILE__)) . '../includes/customily-options-utils.php';
        $this->orders_notifications_table_name = "{$wpdb->prefix}customily_{$this->orders_notifications_table_name}";
    }

    /**
     * @return string|null
     */
    public function get_orders_notifications_count()
    {
        global $wpdb;
        return $wpdb->get_var(
            $wpdb->prepare("SELECT COUNT(*) FROM %i", [$this->orders_notifications_table_name])
        );
    }

    /**
     * Get order notification by order_id
     * @param $order_id
     * @return array|false|object|stdClass|void|null
     */
    public function get_order_notification($order_id)
    {
        global $wpdb;
        try {
            return $wpdb->get_row(
                $wpdb->prepare(
                    "SELECT * FROM %i WHERE order_id = %d",
                    [$this->orders_notifications_table_name, $order_id]
                )
            );
        } catch (Exception $e) {
            Customily_Options_Utils::add_log($e->getTraceAsString());
            return false;
        }
    }

    /**
     * @param $limit
     * @param $offset
     * @return array|false|object|stdClass[]|null
     */
    public function get_orders_notifications($limit, $offset)
    {
        global $wpdb;
        try {
            return $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT * FROM %i LIMIT %d OFFSET %d",
                    [$this->orders_notifications_table_name, $limit, $offset]
                ),
                ARRAY_A
            );
        } catch (Exception $e) {
            Customily_Options_Utils::add_log($e->getTraceAsString());
            return false;
        }
    }

    /**
     * Create or update order_notification
     * @param $order
     * @return bool|int|mysqli_result|resource|null
     */
    public function create_or_update_order_notification($order)
    {
        $order_notification = $this->get_order_notification($order['id']);
        if (empty($order_notification)) {
            $notify_attempts = (int)$order_notification->notify_attempts;
            return $this->update_order_notification(
                $order['id'],
                array(
                    'notify_attempts' => $notify_attempts + 1
                )
            );
        } else {
            return $this->create_order_notification($order);
        }
    }

    /**
     * Create order_notification
     * @param $order
     * @return bool|int|mysqli_result|resource|null
     */
    public function create_order_notification($order)
    {
        global $wpdb;
        $personalization_ids = array();

        foreach ($order['line_items'] as $item) {
            if (isset($item['properties']) && !empty($item['properties'])) {
                foreach ($item['properties'] as $property) {
                    if ($property['name'] == '_customily-personalization-id') {
                        array_push($personalization_ids, $property['value']);
                        break;
                    }
                }
            }
        }

        if (count($personalization_ids) > 0) {
            $notification = array(
                'order_id' => $order['id'],
                'order_number' => $order['name'],
                'personalization_ids' => implode(',', $personalization_ids)
            );

            try {
                return $wpdb->insert($this->orders_notifications_table_name, $notification);
            } catch (Exception $e) {
                Customily_Options_Utils::add_log($e->getTraceAsString());
            }
        }
        return false;
    }

    /**
     * Update order_notification
     * @param $order_id
     * @param $notification
     * @return bool|int|mysqli_result|resource|null
     */
    public function update_order_notification($order_id, $notification)
    {
        global $wpdb;
        try {
            return $wpdb->update(
                $this->orders_notifications_table_name,
                $notification,
                array(
                    'order_id' => $order_id
                )
            );
        } catch (Exception $e) {
            Customily_Options_Utils::add_log($e->getTraceAsString());
            return false;
        }
    }

    /**
     * @param $order_id
     * @return bool|int|mysqli_result|resource|null
     */
    public function delete_order_notification($order_id)
    {
        global $wpdb;
        try {
            return $wpdb->delete(
                $this->orders_notifications_table_name,
                array(
                    'order_id' => $order_id
                )
            );
        } catch (Exception $e) {
            Customily_Options_Utils::add_log($e->getTraceAsString());
            return false;
        }
    }
}