<?php
class Customily_Options_Products_DB
{
    public $post_table_name = 'posts';
    public $meta_table_name = 'postmeta';

    /**
     * Customily_Options_Products_DB constructor.
     */
    public function __construct()
    {
        global $wpdb;
        $this->post_table_name = "{$wpdb->prefix}{$this->post_table_name}";
        $this->meta_table_name = "{$wpdb->prefix}{$this->meta_table_name}";
        require_once plugin_dir_path(dirname(__FILE__)) . '../includes/customily-options-utils.php';
    }

    /**
     * Create product
     * @param $product_info
     * @return int|false
     */
    public function create_product($product_info)
    {
        try {
            if (isset($product_info) && !empty($product_info)) {
                //Create main product
                $slug = sanitize_title($product_info['name']);
                $product = $this->get_variable_product($product_info);
                $product->set_slug($slug);

                if (!empty($product_info['vendor'])) {
                    $product->add_meta_data('pod_vendor', $product_info['vendor']);
                }

                $images_ids = [];

                if (isset($product_info['images']) && count($product_info['images']) > 0) {
                    foreach ($product_info['images'] as $image) {
                        if (isset($image['src'])) {
                            $media_id = Customily_Options_Utils::upload_media($image['src']);
                            if ($media_id) {
                                $images_ids[] = $media_id;
                            }
                        }
                    }
                }

                if (count($images_ids) > 0) {
                    $product->set_image_id($images_ids[0]);
                }

                $attributes = [];

                if (isset($product_info['options']) && count($product_info['options']) > 0) {
                    foreach ($product_info['options'] as $option) {
                        $attr = $this->create_attribute($option['name'], $option['values']);

                        $attribute = new WC_Product_Attribute();
                        $attribute->set_id($attr['attribute_id']);
                        $attribute->set_name($attr['attribute_taxonomy']);
                        $attribute->set_options($option['values']);
                        $attribute->set_visible(1);
                        $attribute->set_variation(1);
                        $attributes[] = $attribute;
                    }
                    $product->set_attributes($attributes);
                }

                //Save main product to get its id
                $product_id = $product->save();

                if (!empty($product_id) && isset($product_info['variants']) && count($product_info['variants']) > 0) {
                    foreach ($product_info['variants'] as $variant) {
                        $variation = new WC_Product_Variation();
                        $variation->set_name($variant['title']);
                        $variation->set_price($variant['price']);
                        $variation->set_regular_price($variant['regular_price'] ?? $variant['price']);
                        $variation->set_parent_id($product_id);

                        if (isset($variant['image']) && !empty($variant['image'])) {
                            $media_id = Customily_Options_Utils::upload_media($variant['image']);
                            if ($media_id) {
                                $variation->set_image_id($media_id);
                            }
                        }

                        if (isset($variant['sku']) && !empty($variant['sku'])) {
                            $variation->set_sku($variant['sku']);
                        }

                        $variation->set_manage_stock($variant['manage_stock'] ?? false);

                        if (isset($variant['stock_quantity'])) {
                            $variation->set_stock_quantity($variant['stock_quantity']);
                        }

                        $variation->set_stock_status('in_stock');

                        $variation_attrs = [];
                        foreach ($variant['options'] as $option) {
                            $taxonomy = "pa_" . wc_sanitize_taxonomy_name(stripslashes($option['name']));
                            $term = get_term_by('name', $option['value'], $taxonomy);
                            $variation_attrs[$taxonomy] = $term->slug;
                        }

                        $variation->set_attributes($variation_attrs);
                        $variation->save();
                    }
                }

                return $product_id;
            }
        } catch (Exception $ex) {
            Customily_Options_Utils::add_log($ex->getTraceAsString());
        }
        return false;
    }

    /**
     * Update product
     * @param $product_info
     * @return array|false
     */
    public function update_product($product_info)
    {
        try {
            if (
                isset($product_info) &&
                !empty($product_info) &&
                isset($product_info['id']) &&
                $product_info['id'] > 0
            ) {
                $product = wc_get_product($product_info['id']);
                $product->set_name(addslashes($product_info['name']));
                $product->set_status($product_info['published'] ? 'publish' : 'private');
                $product->set_catalog_visibility('visible');
                $product->set_description(addslashes($product_info['description'] ?? ''));

                if (!empty($product_info['vendor'])) {
                    $product->add_meta_data('pod_vendor', $product_info['vendor']);
                }

                if (!empty($product_info['image'])) {
                    $media_id = Customily_Options_Utils::upload_media($product_info['image']);
                    $product->set_image_id($media_id);
                }

                $result = $product->save();

                if (count($product_info['variants']) > 0) {
                    foreach ($product_info['variants'] as $variant) {
                        $variation = wc_get_product($variant['id']);
                        if (!empty($variation)) {
                            if (!empty($variant['sku'])) {
                                $variation->set_sku($variant['sku']);
                            }

                            if (!empty($variant['price'])) {
                                $variation->set_price($variant['price']);
                                $variation->set_regular_price($variant['regular_price'] ?? $variant['price']);
                            }

                            $variation->save();
                        }
                    }
                }

                return $result;
            }
        } catch (Exception $ex) {
            Customily_Options_Utils::add_log($ex->getTraceAsString());
        }
        return false;
    }

    /**
     * Create demo product
     * @param $product_info
     * @return int|false
     */
    public function create_demo_product($product_info)
    {
        $product_info['options'] = [
            [
                'name' => 'Style',
                'values' => ['Text', 'Image']
            ]
        ];

        $product_info['variants'] = [
            [
                'title' => "{$product_info['name']} - Text",
                'price' => 10.00,
                'options' => [
                    [
                        'name' => 'Style',
                        'value' => 'Text'
                    ]
                ]
            ],
            [
                'title' => "{$product_info['name']} - Image",
                'price' => 10.00,
                'options' => [
                    [
                        'name' => 'Style',
                        'value' => 'Image'
                    ]
                ]
            ]
        ];

        $product_info['images'] = [
            [
                'src' => esc_url(CUSTOMILY_OPTIONS_ASSETS_ROOT_URL) . '/static/images/01.jpg'
            ],
            [
                'src' => esc_url(CUSTOMILY_OPTIONS_ASSETS_ROOT_URL) . '/static/images/02.jpg'
            ]
        ];

        return $this->create_product($product_info);
    }

    /**
     * Create variable product
     * @param $product_info
     * @return WC_Product_Variable
     */
    private function get_variable_product($product_info) {
        $product = new WC_Product_Variable();
        $product->set_name(addslashes($product_info['name']));
        $product->set_status($product_info['published'] ? 'publish' : 'private');
        $product->set_catalog_visibility('visible');
        $product->set_description($product_info['description'] ?? '');
        $product->set_description(addslashes($product->get_description()));
        $product->set_sku($product_info['sku'] ?? '');
        return $product;
    }

    /**
     * @param $search_string
     * @return mixed
     */
    public function get_products_count($search_string, $ids) {
        global $wpdb;
        $query = "SELECT COUNT(ID) FROM {$this->post_table_name} WHERE post_type = 'product'" .
            " AND post_status IN ('private', 'publish')";
        $query = $this->build_products_query($query, $search_string, $ids);
        return $wpdb->get_var($wpdb->prepare($query));
    }

    /**
     * @param $search_string
     * @param $ids
     * @param $limit
     * @param $offset
     * @return array|object|stdClass[]|null
     */
    public function get_products_posts($search_string, $ids, $limit, $offset)
    {
        global $wpdb;
        $query = "SELECT ID, post_name FROM {$this->post_table_name} WHERE post_type = 'product'" .
            " AND post_status IN ('private', 'publish')";
        $query = $this->build_products_query($query, $search_string, $ids);
        $query .= " ORDER BY ID ASC LIMIT $limit OFFSET $offset";
        return $wpdb->get_results($wpdb->prepare($query));
    }

    /**
     * @param $search_string
     * @param $limit
     * @param $offset
     * @return array
     */
    public function get_products($search_string, $ids, $limit, $offset)
    {
        $posts = $this->get_products_posts($search_string, $ids, $limit, $offset);
        $result = [];
        if (count($posts) > 0) {
            foreach ($posts as $post) {
                try {
                    $result[] = $this->build_product_item($post->ID);
                } catch (Exception $e) {
                    Customily_Options_Utils::add_log(
                        "Customily Wordpress: Error building product item {$post->ID}," .
                        " handle: {$post->post_name}"
                    );
                    Customily_Options_Utils::add_log($e->getTraceAsString());
                }
            }
        }
        return $result;
    }

    /**
     * @param $ids
     * @return mixed
     */
    public function get_products_config_count($ids) {
        global $wpdb;
        $query = "SELECT COUNT(p.ID) FROM %i AS p";
        $query = $this->build_products_config_query($query, $ids);
        return $wpdb->get_var($wpdb->prepare($query, [$this->post_table_name]));
    }

    /**
     * @param $limit
     * @param $offset
     * @param $ids
     * @param $only_post_meta
     * @return array
     */
    public function get_products_config($limit, $offset, $ids, $only_post_meta) {
        global $wpdb;
        $result = [];
        if ($only_post_meta == '1') {
            $query = "SELECT pm.post_id, pm.meta_value FROM %i AS pm";
            $query .= " WHERE pm.meta_key = '_cl_admin_state'";
            $query .= " ORDER BY pm.post_id ASC LIMIT %d OFFSET %d";
            $posts = $wpdb->get_results($wpdb->prepare($query, [$this->post_table_name, $limit, $offset]));
            $result = [
                'result' => $posts,
                'count' => count($posts)
            ];
        } else {
            $query = "SELECT p.ID, p.post_title, pm.meta_value AS meta_data FROM {$this->post_table_name} AS p";
            $query = $this->build_products_config_query($query, $ids);
            $query .= " ORDER BY p.ID ASC LIMIT {$limit} OFFSET {$offset}";
            $posts = $wpdb->get_results($wpdb->prepare($query));

            if (count($posts) > 0) {
                foreach ($posts as $post) {
                    $config = json_decode($post->meta_data, true);
                    if (
                        isset($config) && isset($config['is_on']) &&
                        $config['is_on'] && !empty($config['fields'])
                    ) {
                        $options = [];
                        $index = 0;
                        foreach ($config['fields'] as $field) {
                            $field_type = $this->get_field_type($field);

                            $option = [
                                'type' => $field_type,
                                'label' => $field['label'] ?? '',
                                'required' => $field['required'] ?? false,
                                'help_text' => $field['helpText'] ?? ''
                            ];

                            if (isset($field['sort_id'])) {
                                $option['id'] = $field['sort_id'];
                                $option['sort_id'] = $field['sort_id'] + 1;
                            } else {
                                $option['id'] = $index;
                                $option['sort_id'] = $index + 1;
                                Customily_Options_Utils::add_log("Sort id not found for product {$post->post_title}, field: {$field['label']}");
                            }

                            if ($field_type == 'Image Upload') {
                                if (isset($field['functions']) && isset($field['functions'][0]) && isset($field['functions'][0]['image_id'])) {
                                    $option['file_upload_image_id'] = $field['functions'][0]['image_id'];
                                }
                                $option['button_text'] = $field['button_text'] ?? '';
                            } else if ($field_type == 'Vector Upload') {
                                if (isset($field['functions']) && isset($field['functions'][0]) && isset($field['functions'][0]['vector_id'])) {
                                    $option['file_upload_vector_id'] = $field['functions'][0]['vector_id'];
                                }
                                $option['button_text'] = $field['button_text'] ?? '';

                            } else {
                                if ($field_type == 'Text Input') {
                                    $option['is_textarea'] = $field['is_textarea'] ?? false;
                                    $option['placeholder'] = $field['placeholder'] ?? '';
                                    $option['initial_value'] = $field['value'] ?? '';

                                    if (isset($field['max_length']) && is_numeric($field['max_length'])) {
                                        $option['max_length'] = $field['max_length'];
                                    }

                                    if ($option['is_textarea']) {
                                        if ($field['max_lines'] && is_numeric($field['max_lines'])) {
                                            $option['max_lines'] = $field['max_lines'];
                                        }
                                    }
                                }

                                if (isset($field['functions'])) {
                                    $option['functions'] = $this->get_functions($field['functions']);
                                }

                                if (isset($field['options']) && count($field['options']) > 0) {
                                    $values = [];

                                    $map_function = function ($el) {
                                        return $el['id'];
                                    };

                                    $plus = min(array_map($map_function, $field['options'])) > 0 ? 0 : 1;
                                    $minus = $plus == 1 ? 0 : 1;

                                    foreach ($field['options'] as $o) {
                                        $value = [
                                            'id' => $o['id'] - $minus,
                                            'sort_id' => (int)$o['id'] + $plus
                                        ];

                                        if (isset($o['value'])) {
                                            $value['value'] = $o['value'];
                                        }

                                        if (isset($o['tooltip'])) {
                                            $value['tooltip'] = $o['tooltip'];
                                            $value['value'] = $o['tooltip'];
                                        }

                                        if (isset($o['bg_color'])) {
                                            $value['bg_color'] = $o['bg_color'];
                                        }

                                        if (isset($o['image_src'])) {
                                            $value['thumb_image'] = $this->get_thumbnail_url($o['image_src']);
                                        }

                                        $value = $this->get_value_fields(
                                            $value,
                                            $o,
                                            $option['functions'],
                                            $value['sort_id']
                                        );
                                        $value['selected'] = $o['selected'] ?? false;
                                        $values[] = $value;
                                    }
                                    if (count($values) > 0) {
                                        $option['values'] = $values;
                                    }
                                }
                            }

                            if (isset($field['conditions']) && count($field['conditions']) > 0) {
                                $conditions = $this->get_conditions($field['conditions']);
                                $option['variation_conditions'] = $conditions;

                                $conditions = $this->get_conditions($field['conditions'], 'options', $config['fields']);
                                $option['conditions'] = $conditions;
                            }
                            $options[] = $option;
                            $index++;
                        }

                        $variations = [];
                        $position = 0;
                        foreach ($config['variations'] as $v) {
                            $position++;

                            $variation = [
                                'name' => ucfirst(str_replace('pa_', '', $v['name'])),
                                'position' => $position,
                                'functions' => isset($v['functions']) ? $this->get_functions($v['functions'], 0) : []
                            ];

                            $values = [];
                            $index = 0;

                            foreach ($v['terms'] as $term) {
                                $index++;
                                $value = [
                                    'value' => ucfirst($term['name'] ?? $term['slug'])
                                ];

                                $value = $this->get_value_fields($value, $term, $variation['functions'], $index);
                                $values[] = $value;
                            }

                            $variation['values'] = $values;
                            $variations[] = $variation;
                        }

                        $result[] = [
                            'product_id' => $post->ID,
                            'template_id' => $config['initial_cid'],
                            'name' => $post->post_title,
                            'options' => $options,
                            'variations' => $variations
                        ];
                    }
                }
            }
        }

        return $result;
    }

    public function get_products_old_config($ids) {
        $result = [];
        $product_ids = explode(',', $ids);
        foreach ($product_ids as $product_id) {
            $metadata = get_post_meta($product_id, '_cl_admin_state', true);
            $metadata = preg_replace("/\r|\n/", "", $metadata);
            $metadata = str_replace("  ", "", $metadata);
            $metadata = str_replace('": ', '":', $metadata);
            $result[] = array(
                'product_id' => $product_id,
                'config' => $metadata
            );
        }
        return $result;
    }

    /**
     * @param $search_string
     * @param $ids
     * @param $limit
     * @param $offset
     */
    public function sync_products($search_string, $ids, $limit, $offset)
    {
        $posts = $this->get_products_posts($search_string, $ids, $limit, $offset);
        $result = [];
        if (count($posts) > 0) {
            foreach ($posts as $post) {
                $product = wc_get_product($post->ID);
                $product->set_description("{$product->get_description()} ");
                $product->save();
                $result[] = [
                    'product_id' => $post->ID,
                    'handle' => $post->post_name
                ];
            }
        }
        return $result;
    }

    /**
     * Build product
     * @param $product_id
     * @param false $include_images
     * @param bool $include_variants
     * @param bool $include_description
     * @return array
     */
    public function build_product_item(
        $product_id,
        $include_images = false,
        $include_variants = true,
        $include_description = true
    )
    {
        $product = wc_get_product($product_id);

        $item = [];
        $item['id'] = $product_id;
        $item['title'] = $product->get_name();
        $item['handle'] = $product->get_slug();
        $item['published'] = $product->get_status() == 'publish';
        $item['body_html'] = $include_description ? $product->get_description() : '';
        $item['image']['src'] = '';

        if ($product->meta_exists('pod_vendor')) {
            $item['vendor'] = $product->get_meta('pod_vendor');
        } else {
            $item['vendor'] = 'No vendor';
        }

        $thumbnail = get_the_post_thumbnail_url($product_id, [300, 300]);
        if (isset($thumbnail) && !empty($thumbnail)) {
            $item['image']['product_id'] = $product_id;
            $item['image']['src'] .= $thumbnail;
            $item['image']['width'] = $item['image']['height'] = 300;
        }

        if ($include_images) {
            $item['images'] = $this->get_product_images($product_id);
        }

        if ($product->is_type('variable')) {

            $product = new WC_Product_Variable($product_id);
            $attributes = $product->get_attributes();
            $variation_options = [];

            $index = 1;
            /* @var WC_Product_Attribute $attribute */
            foreach ($attributes as $attribute) {
                if ($attribute->get_variation()) {
                    $position = $attribute->get_position() + $index;
                    $name = ucfirst(str_replace('pa_', '', $attribute->get_name()));
                    $terms = $attribute->get_terms();
                    $values = [];
                    $variation_terms = [];

                    if (isset($terms) && count($terms) > 0) {
                        foreach ($terms as $term) {
                            /* @var WP_Term $term */
                            $values[] = $term->name;
                            $variation_terms[] = [
                                'slug' => urldecode($term->slug),
                                'name' => $term->name
                            ];
                        }
                    } else {
                        $values = $attribute->get_slugs();
                    }

                    $variation_options[] = [
                        'id' => $product_id + 1000000 + $position,
                        'product_id' => $product_id,
                        'name' => $name,
                        'position' => $position,
                        'values' => $values,
                        'terms' => $variation_terms
                    ];
                    $index++;
                }
            }

            $variation_options = Customily_Options_Utils::shell_sort($variation_options, 'position');
            for ($i = 0; $i < count($variation_options); $i++) {
                $variation_options[$i]['position'] = $i + 1;
            }

            $variants = [];

            if ($include_variants) {
                $product_variable = new WC_Product_Variable($product_id);
                $variations_data = $product_variable->get_children();

                foreach ($variations_data as $variation_id) {
                    $variant = new WC_Product_Variation($variation_id);
                    $attributes = $variant->get_attributes();
                    $options = [];
                    foreach ($attributes as $key => $value) {
                        $options[] = str_replace('+', '_plus_', $value);
                    }
                    $variants[] = [
                        'id' => $variant->get_id(),
                        'product_id' => $variant->get_parent_id(),
                        'title' => ucwords(str_replace('-', ' ', implode(' / ', $variant->get_variation_attributes()))),
                        'price' => $variant->get_price(),
                        'sku' => $variant->get_sku(),
                        'position' => $variant->get_menu_order(),
                        'option1' => isset($options[0]) ? str_replace('_plus_', '+', urldecode($options[0])) : '',
                        'option2' => isset($options[1]) ? str_replace('_plus_', '+', urldecode($options[1])) : '',
                        'option3' => isset($options[2]) ? str_replace('_plus_', '+', urldecode($options[2])) : '',
                        'option4' => isset($options[3]) ? str_replace('_plus_', '+', urldecode($options[3])) : '',
                        'option5' => isset($options[4]) ? str_replace('_plus_', '+', urldecode($options[4])) : '',
                        'option6' => isset($options[5]) ? str_replace('_plus_', '+', urldecode($options[5])) : '',
                        'option7' => isset($options[6]) ? str_replace('_plus_', '+', urldecode($options[6])) : '',
                        'option8' => isset($options[7]) ? str_replace('_plus_', '+', urldecode($options[7])) : '',
                        'option9' => isset($options[8]) ? str_replace('_plus_', '+', urldecode($options[8])) : '',
                        'option10' => isset($options[9]) ? str_replace('_plus_', '+', urldecode($options[9])) : '',
                        'option11' => isset($options[10]) ? str_replace('_plus', '+', urldecode($options[10])) : '',
                        'option12' => isset($options[11]) ? str_replace('_plus_', '+', urldecode($options[11])) : '',
                        'option13' => isset($options[12]) ? str_replace('_plus_', '+', urldecode($options[12])) : '',
                        'option14' => isset($options[13]) ? str_replace('_plus_', '+', urldecode($options[13])) : '',
                        'option15' => isset($options[14]) ? str_replace('_plus_', '+', urldecode($options[14])) : '',
                        'created_at' => $variant->get_date_created()->date('Y-m-d H:m:s'),
                        'updated_at' => $variant->get_date_modified()->date('Y-m-d H:m:s')
                    ];
                }
            }

            $default_variant_id = 0;
            foreach ($product->get_available_variations() as $variation_values ) {
                $is_default_variation = false;
                foreach ($variation_values['attributes'] as $key => $attribute_value ) {
                    $attribute_name = str_replace('attribute_', '', $key);
                    $default_value = $product->get_variation_default_attribute($attribute_name);
                    if ($default_value == $attribute_value) {
                        $is_default_variation = true;
                    } else {
                        $is_default_variation = false;
                        break;
                    }
                }
                if ($is_default_variation) {
                    $default_variant_id = $variation_values['variation_id'];
                    break;
                }
            }

            $item['options'] = $variation_options;
            $item['variants'] = $variants;
            $item['updated_at'] = $product->get_date_modified()->getTimestamp();
            $item['available'] = $product->is_in_stock();
            $item['default_variant_id'] = $default_variant_id;
        } else {
            $item['price'] = number_format(floatval($product->get_price()), 2, '.', '');
        }

        return $item;
    }

    /**
     * @param $product_id
     * @return array
     */
    public function get_product_images($product_id)
    {
        $images = [];
        $product = new WC_Product($product_id);

        // main images
        $images[] = wp_get_attachment_url($product->get_image_id());
        $image_ids = $product->get_gallery_image_ids();

        // gallery images
        foreach ($image_ids as $image_id) {
            $images[] = wp_get_attachment_url($image_id);
        }

        return $images;
    }

    /**
     * @param $query
     * @param $search_string
     * @param $ids
     * @return mixed|string
     */
    private function build_products_query($query, $search_string, $ids)
    {
        $result = $query;
        if (!empty($search_string)) {
            $search_string = strtolower($search_string);
            $result .= " AND LOWER(post_title) LIKE '%$search_string%'";
        }
        if (!empty($ids)) {
            $result .= " AND ID IN ({$ids})";
        }
        // id of the first synchronized product
        $first_product_id_synchronized = get_option('cl_first_product_id_synchronized');
        if (!empty($first_product_id_synchronized)) {
            $result .= " AND ID >= {$first_product_id_synchronized}";
        }
        // id of the last synchronized product
        $last_product_id_synchronized = get_option('cl_last_product_id_synchronized');
        if (!empty($last_product_id_synchronized)) {
            $result .= " AND ID <= {$last_product_id_synchronized}";
        }

        return $result;
    }

    /**
     * @param $query
     * @param $ids
     * @return string
     */
    private function build_products_config_query($query, $ids)
    {
        $result = $query;
        $result .= " INNER JOIN {$this->meta_table_name} AS pm ON p.ID = pm.post_id" .
            " WHERE p.post_type = 'product'" .
            " AND p.post_status IN ('private', 'publish')" .
            " AND pm.meta_key = '_cl_admin_state'"; // AND pm.meta_value LIKE '%\"is_on\": true%'
        if (!empty($ids)) {
            $result .= " AND p.ID IN ({$ids})";
        }
        // id of the first migrated product
        $first_product_id_migrated = get_option('cl_first_product_id_migrated');
        if (!empty($first_product_id_migrated)) {
            $result .= " AND p.ID >= {$first_product_id_migrated}";
        }
        // id of the last migrated product
        $last_product_id_migrated = get_option('cl_last_product_id_migrated');
        if (!empty($last_product_id_sync)) {
            $result .= " AND p.ID <= {$last_product_id_migrated}";
        }
        return $result;
    }

    /**
     * @param $thumbnail_url
     * @return mixed|string
     */
    private function get_thumbnail_url($thumbnail_url) {
        global $wp_filesystem;
        if (!$wp_filesystem) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
            WP_Filesystem();
        }

        $upload_url = trailingslashit(wp_upload_dir()['baseurl']);
        $upload_url = "{$upload_url}customily/swatches";

        if (substr_count($thumbnail_url, 'data:image/png;base64') > 0 ||
            substr_count($thumbnail_url, 'data:image/jpeg;base64') > 0) {

            $data = $thumbnail_url;
            $upload_dir = trailingslashit(wp_upload_dir()['basedir']);
            $upload_dir = str_replace('\\', '/', $upload_dir);
            $upload_dir = "{$upload_dir}customily/swatches";

            list($type, $data) = explode(';', $data);
            list(, $data) = explode(',', $data);
            $data = base64_decode($data);

            list(, $extension) = explode('/', $type);

            if (!file_exists($upload_dir)) {
                $wp_filesystem->mkdir($upload_dir, 0777, true);
            }

            $image_id = uniqid('', true);
            $image_id = str_replace('.', '', $image_id);
            $image_dir = "{$upload_dir}/{$image_id}.{$extension}";
            $image_url = "{$upload_url}/{$image_id}.{$extension}";

            $result = $wp_filesystem->put_contents($image_dir, $data);
            if ($result !== false) {
                Customily_Options_Utils::add_log("Converting base64 to image, url: {$image_url}");
                return $image_url;
            }
        } else {
            $index = strpos($thumbnail_url, '/uploads/customily/swatches/');
            if ($index > -1) {
                $filename = substr($thumbnail_url, strrpos($thumbnail_url, '/'), strlen($thumbnail_url));
                Customily_Options_Utils::add_log("Fixing image old url: {$thumbnail_url}, new url: {$upload_url}{$filename}");
                return "{$upload_url}{$filename}";
            }
        }
        return $thumbnail_url;
    }

    /**
     * @param $field
     * @return string
     */
    private function get_field_type($field) {
        $result = '';
        switch ($field['type']) {
            case 'input_text':
                $result = 'Text Input';
                break;
            case 'dropdown':
                $result = 'Dropdown';
                break;
            case 'swatch':
                $result = 'Swatch';
                break;
            case 'input_file':
                $result = 'Image Upload';
                break;
            case 'input_vector':
                $result = 'Vector Upload';
                break;
        }
        return $result;
    }

    /**
     * @param $function
     * @return mixed|string
     */
    private function get_function_type($function) {
        $result = '';
        switch ($function) {
            case 'text':
            case 'color':
            case 'font':
            case 'image':
            case 'vector':
            case 'product':
                $result = $function;
                break;
            case 'image_color':
                $result = 'imageColor';
                break;
            case 'vector_color':
                $result = 'vectorColor';
                break;
        }
        return $result;
    }

    /**
     * @param $value
     * @param $option
     * @param $functions
     * @param $index
     * @return mixed
     */
    private function get_value_fields($value, $option, $functions, $index) {
        if (isset($option['font_id']) && !empty($option['font_id'])) {
            $value['font_id'] = $option['font_id'];
        } else if ($this->find_function('font', $functions)) {
            $value['font_id'] = $index;
        }

        if (isset($option['color_id']) && !empty($option['color_id'])) {
            $value['color_id'] = $option['color_id'];
        } else if ($this->find_function('color', $functions)) {
            $value['color_id'] = $index;
        }

        if (isset($option['image_id']) && !empty($option['image_id'])) {
            $value['image_id'] = $option['image_id'];
        } else if ($this->find_function('image', $functions)) {
            $value['image_id'] = $index;
        }

        if (isset($option['image_color_id']) && !empty($option['image_color_id'])) {
            $value['color_id'] = $option['image_color_id'];
        } else if ($this->find_function('imageColor', $functions)) {
            $value['color_id'] = $index;
        }

        if (isset($option['vector_position']) && !empty($option['vector_position'])) {
            $value['vector_id'] = $option['vector_position'];
        } else if ($this->find_function('vector', $functions)) {
            $value['vector_id'] = $index;
        }

        if (isset($option['vector_color_id']) && !empty($option['vector_color_id'])) {
            $value['color_id'] = $option['vector_color_id'];
        } else if ($this->find_function('vectorColor', $functions)) {
            $value['color_id'] = $index;
        }

        if (isset($option['product_id'])) {
            $value['product_id'] = $option['product_id'];
        }

        return $value;
    }

    /**
     * @param $functions
     * @param int $plus
     * @return array
     */
    private function get_functions($functions, $plus = 1) {
        $result = [];

        foreach ($functions as $function) {
            $f = $function;
            $f['id'] = $function['id'] + $plus;
            $f['type'] = $this->get_function_type($function['type']);
            $result[] = $f;
        }

        return $result;
    }

    /**
     * @param $conditions_array
     * @param string $type
     * @param array $fields
     * @return array
     */
    private function get_conditions($conditions_array, $type = 'variations', $fields = []) {
        if ($type == 'options') {
            $filter = function ($el) {
                return !isset($el['is_variant']) || !$el['is_variant'];
            };
        } else {
            $filter = function ($el) {
                return isset($el['is_variant']) && $el['is_variant'] == '1';
            };
        }

        $conditions = array_filter($conditions_array, $filter);
        $index = 0;
        $result = [];

        foreach ($conditions as $condition) {
            $index++;
            $cond = [
                'id' => $index
            ];

            if (isset($condition['watch_field'])) {
                if ($type == 'options' && is_numeric($condition['watch_field'])) {
                    $sort_id = $this->find_field_sort_id($condition['watch_field'], $fields);
                    if ($sort_id > -1) {
                        $cond['watch_option'] = $sort_id;
                    } else {
                        $cond['watch_option'] = $condition['watch_field'] - 1;
                    }
                } else {
                    $cond['watch_option'] = $condition['watch_field'];
                }
            }

            if (isset($condition['desired_value'])) {
                if ($type == 'options' && is_numeric($condition['desired_value'])) {
                    $map_function = function ($el) {
                        return $el['id'];
                    };
                    $options = $this->find_field_options($condition['watch_field'], $fields);
                    $minus = min(array_map($map_function, $options)) > 0 ? 1 : 0;
                    $cond['desired_value'] = $condition['desired_value'] - $minus;
                } else {
                    $cond['desired_value'] = $condition['desired_value'];
                }
            }

            if ($index == 1) {
                if (isset($condition['action']) && !empty($condition['action'])) {
                    $action = $condition['action'];
                } else {
                    $c = $conditions_array[0];
                    if (isset($c['action']) && !empty($c['action'])) {
                        $action = $c['action'];
                    } else {
                        $action = 'show';
                    }
                }
                $cond['action'] = $action;
            } else {
                if (isset($condition['combination_operator']) && !empty($condition['combination_operator'])) {
                    $cond['combination_operator'] = $condition['combination_operator'];
                } else {
                    $cond['combination_operator'] = 'or';
                }
            }
            $result[] = $cond;
        }
        return $result;
    }

    /**
     * @param $type
     * @param $functions
     * @return bool
     */
    private function find_function($type, $functions) {
        foreach ($functions as $function) {
            if ($function['type'] == $type) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param $id
     * @param $fields
     * @return int|mixed
     */
    private function find_field_sort_id($id, $fields) {
        $index = 0;
        foreach ($fields as $field) {
            if ($field['id'] == $id) {
                return $field['sort_id'] ?? $index;
            }
            $index++;
        }
        return -1;
    }

    /**
     * @param $id
     * @param $fields
     * @return array|mixed
     */
    private function find_field_options($id, $fields) {
        foreach ($fields as $field) {
            if ($field['id'] == $id && isset($field['options'])) {
                return $field['options'];
            }
        }
        return [];
    }

    /**
     * Create attribute
     * @param $name
     * @param $terms
     * @return array
     */
    private function create_attribute($name, $terms)
    {
        // Make sure caches are clean.
        delete_transient('wc_attribute_taxonomies');
        try {
            WC_Cache_Helper::invalidate_cache_group('woocommerce-attributes');
        } catch (Exception $ex) {
            WC_Cache_Helper::incr_cache_prefix('woocommerce-attributes');
        }

        // These are exported as labels, so convert the label to a name if possible first.
        $attribute_labels = wp_list_pluck(wc_get_attribute_taxonomies(), 'attribute_label', 'attribute_name');
        $attribute_name = array_search($name, $attribute_labels, true);

        if (!$attribute_name) {
            $attribute_name = wc_sanitize_taxonomy_name($name);
        }

        $attribute_id = wc_attribute_taxonomy_id_by_name($attribute_name);

        if (!$attribute_id) {
            $taxonomy_name = wc_attribute_taxonomy_name($attribute_name);
            $attribute_id = wc_create_attribute(
                array(
                    'name' => $name,
                    'slug' => $attribute_name,
                    'type' => 'select',
                    'order_by' => 'menu_order',
                    'has_archives' => 0,
                )
            );

            // Register as taxonomy.
            register_taxonomy(
                $taxonomy_name,
                apply_filters("woocommerce_taxonomy_objects_{$taxonomy_name}", array('product')),
                apply_filters(
                    "woocommerce_taxonomy_args_{$taxonomy_name}",
                    array(
                        'labels' => array(
                            'name' => $name,
                        ),
                        'hierarchical' => false,
                        'show_ui' => false,
                        'query_var' => true,
                        'rewrite' => false,
                    )
                )
            );
        }

        $attribute = wc_get_attribute($attribute_id);
        $return = array(
            'attribute_id' => $attribute_id,
            'attribute_name' => $attribute->name,
            'attribute_taxonomy' => $attribute->slug,
            'term_ids' => array(),
            'term_slugs' => array()
        );

        foreach ($terms as $term) {
            if (!term_exists($term, $attribute->slug)) {
                wp_insert_term($term, $attribute->slug);
            }

            $result = get_term_by('name', $term, $attribute->slug);

            $return['term_ids'][] = $result->term_id;
            $return['term_slugs'][] = $result->slug;
        }

        return $return;
    }
}