<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link                    https://www.customily.com
 * @since                   1.0.0
 * @package                 customily-options
 *
 * @wordpress-plugin
 * Plugin Name:             Customily Product Personalizer
 * Plugin URI:              https://www.customily.com
 * Description:             This plugin will let you add personalization fields to your products and link them to Customily Product Personalizer
 * Version:                 1.40.2
 * Author:                  Customily
 * Author URI:              https://github.com/Customily/WooCommerce
 *
 * Requires Plugins:        woocommerce
 * Requires at least:       5.0
 * Tested up to:            6.7
 * WC requires at least:    3.0
 * WC tested up to:         8.0
 * Requires PHP:            7.0
 * Stable tag:              1.40.3
 *
 * License: GPLv3 or later
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

require_once plugin_dir_path(__FILE__) . 'env.php';

const CUSTOMILY_OPTIONS_ENV = 'prod';
const CUSTOMILY_OPTIONS_COOKIE_NAME = 'userToken';
const CUSTOMILY_OPTIONS_PLUGIN_NAME = 'customily-v2';
const CUSTOMILY_OPTIONS_PLUGIN_VERSION = '1.40.3';
const CUSTOMILY_OPTIONS_ORDERITEM_LAST_INDEX = '_customily_orderitem_last_index';
const CUSTOMILY_OPTIONS_PRODUCT_CONNECTED_KEY = '_customily_product_connected';
const CUSTOMILY_OPTIONS_ORDER_CREATION_NOTIFIED = '_customily_order_creation_notified';
const CUSTOMILY_OPTIONS_ORDER_CREATION_NOTIFY_QTY = 20;

require_once plugin_dir_path(__FILE__) . 'includes/customily-options-utils.php';

define('CUSTOMILY_OPTIONS_SITE_URL', Customily_Options_Utils::get_site_url());
define('CUSTOMILY_OPTIONS_ADMIN_URL', admin_url() . '/admin.php?page=customily');

if (!empty($CUSTOMILY_APP_ENV)) {
    define('CUSTOMILY_OPTIONS_CDN_URL', $CUSTOMILY_APP_ENV[CUSTOMILY_OPTIONS_ENV]['cdn_url']);
    define('CUSTOMILY_OPTIONS_S3_ROOT_URL', $CUSTOMILY_APP_ENV[CUSTOMILY_OPTIONS_ENV]['s3_base_url']);
    define('CUSTOMILY_OPTIONS_BACKEND_URL', $CUSTOMILY_APP_ENV[CUSTOMILY_OPTIONS_ENV]['backend_url']);
    define('CUSTOMILY_OPTIONS_APP_BASE_URL', $CUSTOMILY_APP_ENV[CUSTOMILY_OPTIONS_ENV]['api_base_url']);
    define('CUSTOMILY_OPTIONS_ASSETS_ROOT_URL', $CUSTOMILY_APP_ENV[CUSTOMILY_OPTIONS_ENV]['assets_base_url']);
    define('CUSTOMILY_OPTIONS_PREVIEW_API_BASE_URL', $CUSTOMILY_APP_ENV[CUSTOMILY_OPTIONS_ENV]['preview_api_base_url']);
}

function customily_custom_upload_mimes($existing_mimes) {
    // Add webm to the list of mime types.
    $existing_mimes['jpg'] = 'image/jpg';
    $existing_mimes['jpeg'] = 'image/jpeg';
    $existing_mimes['png'] = 'image/png';
    $existing_mimes['heic'] = 'image/heic';
    $existing_mimes['webp'] = 'image/webp';
    // Return the array back to the function with our added mime type.
    return $existing_mimes;
}
add_filter('upload_mimes', 'customily_custom_upload_mimes');

add_action('before_woocommerce_init', function () {
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class)) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
    }

    if (class_exists('\Automattic\WooCommerce\Utilities\FeaturesUtil')) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('cart_checkout_blocks', __FILE__, false);
    }
});

if (!function_exists('wkwc_is_wc_order')) {
    /**
     * Check is post WooCommerce order.
     *
     * @param int $post_id Post id.
     *
     * @return bool $bool True|false.
     */
    function wkwc_is_wc_order($post_id = 0)
    {
        return \Automattic\WooCommerce\Utilities\OrderUtil::get_order_type($post_id) === 'shop_order';
    }
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/customily-options-activator.php
 *
 * @param $network_wide
 */
function activate_customily_options_plugin($network_wide)
{
    require_once plugin_dir_path(__FILE__) . 'includes/customily-options-activator.php';
    $activator = new Customily_Options_Activator();
    $activator->activate($network_wide);
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/customily-options-deactivator.php
 *
 * @param $network_wide
 */
function deactivate_customily_options_plugin($network_wide)
{
    require_once plugin_dir_path(__FILE__) . 'includes/customily-options-deactivator.php';
    $deactivator = new Customily_Options_Deactivator();
    $deactivator->deactivate($network_wide);
}

register_activation_hook(__FILE__, 'activate_customily_options_plugin');
register_deactivation_hook(__FILE__, 'deactivate_customily_options_plugin');

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require_once plugin_dir_path(__FILE__) . 'includes/customily-options.php';

/**
 * The class responsible for defining all cron schedules task
 */
require_once plugin_dir_path(__FILE__) . 'admin/cron/customily-options-cron-schedules.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.1
 */
function run_customily_plugin()
{
    $plugin = new Customily_Options();
    $plugin->run();
}

function can_run_customily()
{
    if (
        in_array(
            'woocommerce/woocommerce.php',
            apply_filters('active_plugins', get_option('active_plugins'))
        )
    ) {
        return true;
    }
    if (is_multisite()) {
        $plugins = get_site_option('active_sitewide_plugins');
        if (isset($plugins['woocommerce/woocommerce.php'])) {
            return true;
        }
    }
    return false;
}

/**
 * Check if WooCommerce is active
 **/
if (can_run_customily()) {
    run_customily_plugin();
    if (function_exists('wc_customily_init_cron_schedules')) {
        wc_customily_init_cron_schedules();
    }
}