<?php

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    customily-options
 * @subpackage customily-options/includes
 * @author     Customily
 */
class Customily_Options_Activator
{

    /* @var Customily_Options_Api_Client */
    private $api_client;

    /**
     * Customily_Options_Activator constructor.
     */
    public function __construct()
    {
        require_once 'customily-options-utils.php';
        require_once 'customily-options-api-client.php';
    }

    /**
     * Activate plugin on site network
     *
     * @param $network_wide
     */
    public function activate($network_wide)
    {
        global $wpdb;

        // Check if the plugin is being network-activated or not.
        if ($network_wide) {
            // Retrieve all site IDs from this network (WordPress >= 4.6 provides easy to use functions for that).
            if (function_exists('get_sites') && function_exists('get_current_network_id')) {
                $site_ids = get_sites(array('fields' => 'ids', 'network_id' => get_current_network_id()));
            } else {
                $site_ids = $wpdb->get_col($wpdb->prepare(
                    "SELECT blog_id FROM %i WHERE site_id = %d",
                    [$wpdb->blogs, $wpdb->site_id]
                ));
            }

            // Install the plugin for all these sites.
            foreach ($site_ids as $site_id) {
                switch_to_blog($site_id);
                $this->activate_single_site(Customily_Options_Utils::get_site_url($site_id));
                restore_current_blog();
            }
        } else {
            $this->activate_single_site(CUSTOMILY_OPTIONS_SITE_URL);
        }
    }

    /**
     * Activate plugin on single site
     */
    private function activate_single_site($site_url)
    {
        $remove_www = 'n';
        $this->api_client = new Customily_Options_Api_Client($site_url);
        $result = $this->do_activate_single_site();
        if (!$result || !$result['success']) {
            $remove_www = 'y';
            // remove www. from site url
            $site_url = preg_replace('#^www.#', '', $site_url);
            $this->api_client->set_site_url($site_url);
            $result = $this->do_activate_single_site();
        }

        if (!$result['success']) {
            if (CUSTOMILY_OPTIONS_ENV == 'prod') {
                $message = Customily_Options_Utils::build_installation_error_message($result['error_code']);
                wp_die('<div style="font-size: 16px; text-align: center">' . esc_html($message) . '</div>');
            }
        } else {
            if ($result['message'] != 'already_registered') {
                $this->api_client->initialize_app();
                Customily_Options_Utils::add_log(
                    "Customily Wordpress: plugin activated successfully for shop '{$this->api_client->get_site_url()}'."
                );
            }
            $this->init_config_options($remove_www);
            Customily_Options_Utils::create_table_orders_notifications();
        }
    }

    /**
     * @return array|false|mixed
     */
    private function do_activate_single_site()
    {
        $result = $this->api_client->register_in_app();
        if (!isset($result)) {
            return false;
        }

        return $result;
    }

    /**
     * Initialize plugin options
     * @param $remove_wwww
     */
    private function init_config_options($remove_www = 'n')
    {
        update_option('cl_remove_www_from_url', $remove_www);
        // Change permlinks structre to avoid api call issues
        update_option('permalink_structure', '/%postname%/');
        // Enable/disable customily cron
        update_option('cl_cron_enabled', 'y');
        // Check if products are connected with customily
        update_option('cl_check_products_connected', 'n');
        // Category for products connected with customily
        update_option('cl_products_category', 'Customily');
        // Api client request timeout in seconds
        update_option('cl_api_requests_timeout', 180);
    }
}
