<?php

class Customily_Options_Api_Client
{
    private $api_key;
    private $site_url;
    private $cookie;

    /**
     * Customily_Options_Api_Client constructor.
     * @param $site_url
     */
    public function __construct($site_url)
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/db/customily-options-products-db.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/db/customily-options-settings-db.php';
        require_once 'customily-options-utils.php';

        $this->api_key = CUSTOMILY_OPTIONS_ENV == 'beta' && !empty(get_option('cl_beta_wp_api_key'))
            ? get_option('cl_beta_wp_api_key')
            : get_option('cl_wp_api_key');
        $this->site_url = $site_url;
        $this->cookie = [];
    }

    /**
     * @return mixed
     */
    public function get_site_url()
    {
        return $this->site_url;
    }

    /**
     * @param mixed $site_url
     */
    public function set_site_url($site_url)
    {
        $this->site_url = $site_url;
    }

    /**
     * @return mixed|null
     */
    public function register_in_app()
    {
        $url = CUSTOMILY_OPTIONS_APP_BASE_URL . '/wordpress';
        $options_db = new Customily_Options_Settings_DB();
        $owner = $options_db->get_store_owner_info();

        $headers = array(
            'shop' => $this->site_url,
            'platform' => 'wordpress',
            'owner_name' => $owner['name'],
            'owner_email' => $owner['email']
        );

        if (!empty($this->api_key) && strlen($this->api_key) > 0) {
            $headers['wp_api_key'] = $this->api_key;
            // Not the first installation
            update_option('cl_first_installation', 'n');
        } else {
            // It's the first installation
            update_option('cl_first_installation', 'y');
        }

        $response = $this->wp_post($url, $headers);

        $result = [
            'success' => false,
            'cookie' => null,
            'error_code' => '',
            'message' => ''
        ];

        $body = json_decode($response['data'], true);

        if ((isset($response['success']) && $response['success']) || $response['code'] == 202) {
            if ($response['code'] == 202) {
                $msg = "Shop '{$this->site_url}' is already registered in Customily App.";
                $result['message'] = 'already_registered';
            } else {
                $this->update_cookie($body);
                $this->update_options($body);
                $msg = "Shop '{$this->site_url}' was registered successfully in Customily App.";
                $result['cookie'] = $body['wpCookie'];
            }
            $result['success'] = true;
            delete_option('cl_registration_error_message');
            delete_option('cl_registration_error_details');
        } else {
            $msg = "Error when trying to install Customily App in shop '{$this->site_url}', please contact the Customily Support to fix the problem. \n";
            update_option('cl_registration_error_message', $msg);
            $details = [
                'status_code' => $response['code'],
                'error_code' => $body['errorType'] ?? '',
                'error_message' => $body['message'] ?? '',
                'request' => wp_json_encode($headers),
                'response' => $response['data']
            ];

            $result['error_code'] = $body['errorType'] ?? '';
            $result['message'] = $body['message'] ?? '';

            update_option('cl_registration_error_details', wp_json_encode($details));
        }

        Customily_Options_Utils::add_log($msg);
        return $result;
    }

    /**
     * @return bool|null
     */
    public function initialize_app()
    {
        $url = CUSTOMILY_OPTIONS_APP_BASE_URL . "/wordpress/initialize?shop={$this->site_url}";
        $headers = array('platform' => 'wordpress');
        $cookies = array(
            new WP_Http_Cookie($this->cookie)
        );

        $result = $this->wp_post($url, $headers, '', $cookies);
        if (isset($result['success']) && $result['success']) {
            Customily_Options_Utils::add_log("Customily App initialized successfully in shop '{$this->site_url}'.");
            return true;
        } else {
            Customily_Options_Utils::add_log("Error when trying to initialize Customily App in shop '{$this->site_url}'.");
            Customily_Options_Utils::add_log($result['data']);
        }
        return null;
    }

    /**
     * @param $product_id
     * @param string $post_name
     * @return mixed|null
     */
    public function update_product_hook($product_id)
    {
        if (isset($this->api_key) && !empty($this->api_key)) {
            $url = CUSTOMILY_OPTIONS_APP_BASE_URL . '/wordpress/products/update';
            $products_db = new Customily_Options_Products_DB();
            $body = wp_json_encode($products_db->build_product_item($product_id));
            $result = $this->wp_post($url, $this->get_hook_header($body), $body);
            if (isset($result['success']) && $result['success']) {
                return json_decode($result['data']);
            } else {
                Custom("Error when trying to update product {$product_id}.");
                Customily_Options_Utils::add_log($result['data']);
            }
        }
        return null;
    }

    /**
     * @param $product_id
     * @return mixed|null
     */
    public function delete_product_hook($product_id)
    {
        if (isset($this->api_key) && !empty($this->api_key)) {
            $url = CUSTOMILY_OPTIONS_APP_BASE_URL . '/wordpress/products/delete';
            $body = wp_json_encode(['id' => $product_id]);
            $result = $this->wp_post($url, $this->get_hook_header($body), $body);
            if (isset($result['success']) && $result['success']) {
                return json_decode($result['data']);
            } else {
                Customily_Options_Utils::add_log("Error when trying to delete product {$product_id}.");
                Customily_Options_Utils::add_log($result['data']);
            }
        }
        return null;
    }

    /**
     * @param $order
     * @return string
     */
    public function order_created($order)
    {
        if (isset($this->api_key) && !empty($this->api_key)) {
            $url = CUSTOMILY_OPTIONS_APP_BASE_URL . '/wordpress/orders/create';
            $body = wp_json_encode($order);
            $result = $this->wp_post($url, $this->get_hook_header($body), $body);

            if (isset($result['success']) && $result['success']) {
                return 'y';
            }
        }
        return 'n';
    }

    /**
     * Get shop settings
     *
     * @return mixed|null
     */
    public function get_shop_settings()
    {
        $url = CUSTOMILY_OPTIONS_APP_BASE_URL . "/api/settings?shop=" . $this->site_url;
        $result = $this->wp_get($url, []);
        if ($result['success']) {
            return json_decode($result['data'], true);
        }
        return $this->get_default_shop_settings();
    }

    /**
     * Get unified settings
     *
     * @param $productHandle
     * @return mixed|null
     */
    public function get_unified_settings($productHandle)
    {
        $url = CUSTOMILY_OPTIONS_APP_BASE_URL . "/api/settings/unified/{$productHandle}?shop=" . $this->site_url;
        $result = $this->wp_get($url, []);
        if ($result['success']) {
            return json_decode($result['data'], true);
        }
        return null;
    }

    /**
     * @param $body
     * @return array
     */
    public function get_hook_header($body)
    {
        $hmac = bin2hex(hash_hmac('sha256', $body, $this->api_key, true));
        return array(
            "Content-Type" => "application/json",
            "x-wordpress-hmac-sha256" => $hmac,
            "x-wordpress-shop-domain" => $this->site_url,
            "platform" => "wordpress"
        );
    }

    /**
     * @param $url
     * @param $headers
     * @param string $body
     * @param array $cookies
     * @return array
     */
    public function wp_post($url, $headers, $body = '', $cookies = [])
    {
        return $this->wp_request('POST', $url, $headers, $body, $cookies);
    }

    /**
     * @param $url
     * @param $headers
     * @return array
     */
    public function wp_get($url, $headers)
    {
        return $this->wp_request('GET', $url, $headers);
    }

    /**
     * @param $method
     * @param $url
     * @param $headers
     * @param string $body
     * @param array $cookies
     * @return array
     */
    private function wp_request($method, $url, $headers, $body = '', $cookies = [])
    {
        $result = ['success' => false, 'code', null, 'data' => null];
        $timeout = get_option('cl_api_requests_timeout');
        if (empty($timeout)) {
            $timeout = 180;
        }

        $params = [
            'method' => $method,
            'httpversion' => '1.1',
            'headers' => $headers,
            'timeout' => (int)$timeout
        ];
        if ($method == 'GET') {
            $response = wp_remote_get($url, $params);
        } else {
            $params['body'] = $body;
            $params['cookies'] = $cookies;
            $response = wp_remote_post($url, $params);
        }

        $result['code'] = wp_remote_retrieve_response_code($response);
        $body = wp_remote_retrieve_body($response);

        if (is_wp_error($response)) {
            $result['data'] = $response->get_error_message();
        } else {
            if (isset($result['code']) && $result['code'] == 200) {
                try {
                    $result['success'] = true;
                } catch (\Exception $e) {
                    $result['data'] = $e->getTraceAsString();
                }
            }
            $result['data'] = $body;
        }

        return $result;
    }

    /**
     * @param $options
     */
    private function update_options($options)
    {
        if (isset($options['wpApiKey'])) {
            if (CUSTOMILY_OPTIONS_ENV == 'beta') {
                update_option('cl_beta_wp_api_key', $options['wpApiKey']);
            } else {
                update_option('cl_wp_api_key', $options['wpApiKey']);
            }
        }
    }

    /**
     * @param $values
     */
    private function update_cookie($values)
    {
        if (isset($values['wpCookie']) && isset($values['wpCookieDomain'])) {
            $expires = time() + (86400 * 365 * 2);
            $this->cookie['name'] = CUSTOMILY_OPTIONS_COOKIE_NAME;
            $this->cookie['value'] = $values['wpCookie'];
            $this->cookie['expires'] = $expires;
            $this->cookie['path'] = '/';
            $this->cookie['domain'] = $values['wpCookieDomain'];
            unset($_COOKIE[CUSTOMILY_OPTIONS_COOKIE_NAME]);
            setcookie(CUSTOMILY_OPTIONS_COOKIE_NAME, '', time() - 7200, '/');
            setcookie(
                CUSTOMILY_OPTIONS_COOKIE_NAME,
                $values['wpCookie'],
                $expires,
                '/',
                $values['wpCookieDomain'],
                $values['wpCookieSecure'] ?? false,
                $values['wpCookieHttpOnly'] ?? false
            );
        }
    }

    /**
     * Get default app settings
     * @return true[]
     */
    private function get_default_shop_settings() {
        return array(
            'enabled' => true,
            'debugMode' => true,
            'exportProductionFile' => true,
            'renameProdFile' => false,
            'enableSavePersonalization' => false
        );
    }
}