<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://wwww.customily.com
 * @since      1.0.0
 *
 * @since      1.0.0
 * @package    customily-options
 * @subpackage customily-options/includes
 * @author     Customily
 */
class Customily_Options
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Customily_Options_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     *
     * @since    1.0.0
     */
    public function __construct()
    {
        add_action('admin_init', array($this, 'check_installation'));
        $this->load_dependencies();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    /**
     * Check plugins installation
     */
    public function check_installation()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/customily-options-utils.php';
        Customily_Options_Utils::check_installation();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Customily_Options_Loader. Orchestrates the hooks of the plugin.
     * - Customily_Options_Api_Server. Defines internal api endpoints of the plugin.
     * - Customily_Options_Admin. Defines all hooks for the admin area.
     * - Customily_Options_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {
        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/customily-options-loader.php';

        /**
         * The class responsible for defining all rest api endpoints
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/customily-options-api-server.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/customily-options-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/customily-options-public.php';

        $this->loader = new Customily_Options_Loader();
    }

    /**
     * Register all of the hooks related to the admin area functionality of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {
        $plugin_admin = new Customily_Options_Admin();
        $this->loader->add_action(
            'admin_menu',
            $plugin_admin,
            'wc_customily_admin_menu',
            PHP_INT_MAX
        );
        $this->loader->add_action(
            'transition_post_status',
            $plugin_admin,
            'wc_customily_publish_product',
            PHP_INT_MAX,
            3
        );
        $this->loader->add_action(
            'woocommerce_product_import_inserted_product_object',
            $plugin_admin,
            'wc_customily_product_imported',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_action(
            'before_delete_post',
            $plugin_admin,
            'wc_customily_before_delete_product',
            PHP_INT_MAX
        );
        $this->loader->add_action(
            'woocommerce_admin_order_data_after_order_details',
            $plugin_admin,
            'wc_customily_show_custom_data_in_order',
            PHP_INT_MAX
        );
        $this->loader->add_filter(
            'woocommerce_admin_order_item_thumbnail',
            $plugin_admin,
            'wc_customily_show_custom_thumbnail_on_order',
            PHP_INT_MAX,
            3
        );
        $this->loader->add_filter(
            'woocommerce_rest_check_permissions',
            $plugin_admin,
            'wc_customily_rest_check_permissions',
            PHP_INT_MAX,
            4
        );
        $this->loader->add_filter(
            'woocommerce_rest_check_permissions',
            $plugin_admin,
            'wc_customily_rest_check_permissions',
            PHP_INT_MAX,
            4
        );
        $this->loader->add_action(
            'woocommerce_order_status_changed',
            $plugin_admin,
            'wc_customily_woocommerce_order_status_changed_hook',
            PHP_INT_MAX,
            4
        );
        $this->loader->add_filter(
            'wc_order_statuses',
            $plugin_admin,
            'wc_customily_woocommerce_add_order_statuses',
            PHP_INT_MAX,
            1
        );
        $this->loader->add_action(
            'init',
            $plugin_admin,
            'wc_customily_register_custom_order_status'
        );
    }

    /**
     * Register all of the hooks related to the public-facing functionality of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks()
    {
        $plugin_public = new Customily_Options_Public();
        $this->loader->add_action(
            'woocommerce_before_single_product_summary',
            $plugin_public,
            'wc_customily_insert_assets',
            PHP_INT_MAX
        );
        $this->loader->add_action(
            'woocommerce_single_product_summary',
            $plugin_public,
            'wc_customily_single_product_summary_sibling',
            PHP_INT_MAX
        );
        $this->loader->add_action(
            'woocommerce_before_add_to_cart_button',
            $plugin_public,
            'wc_customily_insert_options_container',
            PHP_INT_MAX
        );
        $this->loader->add_action(
            'woocommerce_after_add_to_cart_button',
            $plugin_public,
            'wc_customily_handle_variable_product_stock_status',
            0
        );
        $this->loader->add_action(
            'woocommerce_product_single_add_to_cart_text',
            $plugin_public,
            'wc_customily_handle_single_product_stock_status',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_filter(
            'woocommerce_add_cart_item_data',
            $plugin_public,
            'wc_customily_add_custom_data_to_cart_item',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_filter(
            'woocommerce_cart_item_price',
            $plugin_public,
            'wc_customily_display_cart_item_price_html',
            PHP_INT_MAX,
            3
        );
        $this->loader->add_action(
            'woocommerce_before_calculate_totals',
            $plugin_public,
            'wc_customily_adjust_cart_items_price',
            PHP_INT_MAX
        );
        $this->loader->add_filter(
            'woocommerce_get_item_data',
            $plugin_public,
            'wc_customily_show_custom_data_in_cart',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_filter(
            'woocommerce_cart_item_thumbnail',
            $plugin_public,
            'wc_customily_cart_item_thumbnail',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_filter(
            'woocommerce_store_api_cart_item_images',
            $plugin_public,
            'wc_customily_store_api_cart_item_images',
            PHP_INT_MAX,
            3
        );
        $this->loader->add_filter(
            'woocommerce_cart_item_name',
            $plugin_public,
            'wc_customily_show_custom_thumbnail_on_checkout',
            PHP_INT_MAX,
            3
        );
        $this->loader->add_action(
            'woocommerce_after_cart',
            $plugin_public,
            'wc_customily_load_cart_script',
            PHP_INT_MAX
        );
        $this->loader->add_action(
            'woocommerce_checkout_create_order_line_item',
            $plugin_public,
            'wc_customily_add_custom_data_to_order_item',
            PHP_INT_MAX,
            4
        );
        $this->loader->add_filter(
            'woocommerce_order_item_name',
            $plugin_public,
            'wc_customily_woocommerce_order_item_name',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_filter(
            'woocommerce_order_item_get_formatted_meta_data',
            $plugin_public,
            'wc_customily_order_item_get_formatted_meta_data',
            PHP_INT_MAX,
            2
        );
        $this->loader->add_action(
            'woocommerce_payment_complete',
            $plugin_public,
            'wc_customily_woocommerce_payment_complete_hook',
            1
        );
        $this->loader->add_filter(
            'woocommerce_single_product_image_gallery_classes',
            $plugin_public,
            'wc_customily_product_image_gallery_classes',
            PHP_INT_MAX,
            1
        );
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        $this->loader->run();
    }
}
